package com.rti.demo;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import com.rti.dds.domain.DomainParticipant;
import com.rti.dds.domain.DomainParticipantFactory;
import com.rti.dds.infrastructure.InstanceHandle_t;
import com.rti.dds.infrastructure.RETCODE_NO_DATA;
import com.rti.dds.infrastructure.ResourceLimitsQosPolicy;
import com.rti.dds.infrastructure.StatusKind;
import com.rti.dds.publication.Publisher;
import com.rti.dds.subscription.DataReader;
import com.rti.dds.subscription.DataReaderAdapter;
import com.rti.dds.subscription.DataReaderListener;
import com.rti.dds.subscription.InstanceStateKind;
import com.rti.dds.subscription.SampleInfo;
import com.rti.dds.subscription.SampleInfoSeq;
import com.rti.dds.subscription.SampleStateKind;
import com.rti.dds.subscription.Subscriber;
import com.rti.dds.subscription.ViewStateKind;
import com.rti.dds.topic.Topic;

import org.eclipse.wb.swt.SWTResourceManager;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.ProgressBar;

public class LegoDemo {

	// GUI Elements;
	static Label touchLabel;
	static Label distanceLabel;
	static ProgressBar distanceProgressBar;
	Button brakeButton;
	Button forwardButton;
	Button reverseButton;
	Button leftButton;
	Button rightButton;
	Text statusMessage;
	
	private Shell shlRtiLegoMindstorms;

	//Connext elements
	DomainParticipant participant = null;
	Publisher publisher = null;
	Topic touchTopic = null;
	Topic distanceTopic = null;
	Topic motorTopic = null;
	MotorCommandDataWriter motorWriter = null;
    MotorCommand motorInstance = new MotorCommand();
    Subscriber subscriber = null;
    DataReaderListener touchListener = null;
    TouchSensorDataReader touchReader = null;
    DataReaderListener distanceListener = null;
    UltraSonicSensorDataReader distanceReader = null;
    private Label lblDriveTheLego;
    private Label lblLegoMinstormsSensor;
    
    	
	public LegoDemo(Display display) {		
		shlRtiLegoMindstorms = new Shell(display);
		shlRtiLegoMindstorms.setImage(SWTResourceManager.getImage(LegoDemo.class, "/com/rti/demo/rti_48x48.png"));
		initUI();
		initConnext();
		setupUIListener();

		shlRtiLegoMindstorms.setText("RTI Lego Mindstorms Demo");
		shlRtiLegoMindstorms.setSize(826, 500);
		
		Label lblNewLabel = new Label(shlRtiLegoMindstorms, SWT.NONE);
		lblNewLabel.setImage(SWTResourceManager.getImage(LegoDemo.class, "/com/rti/demo/mindstorms.png"));
		lblNewLabel.setBounds(10, 22, 282, 372);				
		
		lblDriveTheLego = new Label(shlRtiLegoMindstorms, SWT.NONE);
		lblDriveTheLego.setFont(SWTResourceManager.getFont("Segoe UI", 14, SWT.BOLD));
		lblDriveTheLego.setBounds(318, 194, 459, 27);
		lblDriveTheLego.setText("Drive the Lego Robot using the following buttons:");
		
		lblLegoMinstormsSensor = new Label(shlRtiLegoMindstorms, SWT.NONE);
		lblLegoMinstormsSensor.setFont(SWTResourceManager.getFont("Segoe UI", 14, SWT.BOLD));
		lblLegoMinstormsSensor.setBounds(318, 22, 325, 27);
		lblLegoMinstormsSensor.setText("Lego Mindstorms Sensor Status:");
		
		Label lblBrake = new Label(shlRtiLegoMindstorms, SWT.NONE);
		lblBrake.setAlignment(SWT.CENTER);
		lblBrake.setBounds(318, 335, 100, 15);
		lblBrake.setText("Brake");
		
		Label lblForward = new Label(shlRtiLegoMindstorms, SWT.NONE);
		lblForward.setAlignment(SWT.CENTER);
		lblForward.setBounds(434, 335, 55, 15);
		lblForward.setText("Forward");
		
		Label lblReverse = new Label(shlRtiLegoMindstorms, SWT.NONE);
		lblReverse.setAlignment(SWT.CENTER);
		lblReverse.setBounds(501, 336, 84, 15);
		lblReverse.setText("Reverse");
		
		Label lblTurnLeft = new Label(shlRtiLegoMindstorms, SWT.NONE);
		lblTurnLeft.setAlignment(SWT.CENTER);
		lblTurnLeft.setBounds(591, 336, 100, 15);
		lblTurnLeft.setText("Turn Left");
		
		Label lblTurnRight = new Label(shlRtiLegoMindstorms, SWT.NONE);
		lblTurnRight.setAlignment(SWT.CENTER);
		lblTurnRight.setBounds(697, 335, 99, 15);
		lblTurnRight.setText("Turn Right");
		
		distanceProgressBar = new ProgressBar(shlRtiLegoMindstorms, SWT.NONE);
		distanceProgressBar.setBounds(366, 148, 249, 17);
		
		Label label = new Label(shlRtiLegoMindstorms, SWT.NONE);
		label.setBounds(363, 171, 26, 15);
		label.setText("0");
		
		Label label_1 = new Label(shlRtiLegoMindstorms, SWT.NONE);
		label_1.setText("255");
		label_1.setBounds(606, 171, 37, 15);

		shlRtiLegoMindstorms.open();

		while (!shlRtiLegoMindstorms.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
		
		//Connext cleanup
		if(participant != null) {
            participant.delete_contained_entities();

            DomainParticipantFactory.TheParticipantFactory.
                delete_participant(participant);
        }
	}

	public void updateTouchLabel(boolean pressed) {
		if(touchLabel != null) {
			if(pressed)
				touchLabel.setText("Touch Sensor Pressed: TRUE!");
			else 
				touchLabel.setText("Touch Sensor Pressed: False");
		}
	}
	
	public void setupUIListener() {
		Listener buttonListener = new Listener() {		
			public void handleEvent(Event event) {
				if (event.widget == brakeButton) {
					statusMessage.append("Brake button pressed.\n");
					/* Write data */
					motorInstance.value = 0;
		            motorWriter.write(motorInstance, InstanceHandle_t.HANDLE_NIL);
		            
				} else if (event.widget == forwardButton) {
					statusMessage.append("Forward button pressed.\n");	
					motorInstance.value = 1;
		            motorWriter.write(motorInstance, InstanceHandle_t.HANDLE_NIL);
				} else if (event.widget == reverseButton) {
					statusMessage.append("Reverse button pressed.\n");
					motorInstance.value = 2;
		            motorWriter.write(motorInstance, InstanceHandle_t.HANDLE_NIL);
				} else if (event.widget == leftButton) {
					statusMessage.append("Left button pressed.\n");
					motorInstance.value = 3;
		            motorWriter.write(motorInstance, InstanceHandle_t.HANDLE_NIL);
				} else if (event.widget == rightButton) {
					statusMessage.append("Right button pressed.\n");
					motorInstance.value = 4;
		            motorWriter.write(motorInstance, InstanceHandle_t.HANDLE_NIL);
				}
				
			}
		};
		
		brakeButton.addListener(SWT.Selection, buttonListener);
		forwardButton.addListener(SWT.Selection, buttonListener);
		reverseButton.addListener(SWT.Selection, buttonListener);
		leftButton.addListener(SWT.Selection, buttonListener);
		rightButton.addListener(SWT.Selection, buttonListener);
	}
	
	public void initConnext() {
		try {
            // --- Create participant --- //
    
            /* To customize participant QoS, use
               the configuration file
               USER_QOS_PROFILES.xml */
    
            participant = DomainParticipantFactory.TheParticipantFactory.
                create_participant(
                    0, DomainParticipantFactory.PARTICIPANT_QOS_DEFAULT,
                    null /* listener */, StatusKind.STATUS_MASK_NONE);
            if (participant == null) {
                statusMessage.append("create_participant error\n");
                return;
            }        
                    
            // --- Create publisher --- //
    
            /* To customize publisher QoS, use
               the configuration file USER_QOS_PROFILES.xml */
    
            publisher = participant.create_publisher(
                DomainParticipant.PUBLISHER_QOS_DEFAULT, null /* listener */,
                StatusKind.STATUS_MASK_NONE);
            if (publisher == null) {
            	statusMessage.append("create_publisher error\n");
                return;
            }                   
                
            
            subscriber = participant.create_subscriber(
            		DomainParticipant.SUBSCRIBER_QOS_DEFAULT, null /* listener */,
            			StatusKind.STATUS_MASK_NONE);
			if (subscriber == null) {
				statusMessage.append("create_subscriber error\n");
			    return;
			}     
        
            // --- Create topic --- //

            /* Register type before creating topic */
            String motorTypeName = MotorCommandTypeSupport.get_type_name();
            MotorCommandTypeSupport.register_type(participant, motorTypeName);
    
            /* To customize topic QoS, use
               the configuration file USER_QOS_PROFILES.xml */
    
            motorTopic = participant.create_topic(
                "MotorTopic",
                motorTypeName, DomainParticipant.TOPIC_QOS_DEFAULT,
                null /* listener */, StatusKind.STATUS_MASK_NONE);
            if (motorTopic == null) {
            	statusMessage.append("create_topic error\n");
                return;
            }           
                
            // --- Create writer --- //
    
            /* To customize data writer QoS, use
               the configuration file USER_QOS_PROFILES.xml */
    
            motorWriter = (MotorCommandDataWriter)
                publisher.create_datawriter(
                    motorTopic, Publisher.DATAWRITER_QOS_DEFAULT,
                    null /* listener */, StatusKind.STATUS_MASK_NONE);
            if (motorWriter == null) {
                statusMessage.append("Create  Motor Data Writer error\n");
                return;
            }           
                                        

            /* Create data sample for writing */
            motorInstance = new MotorCommand();
            
            
            // --- Create topic --- //
            
            /* Register type before creating topic */
            String touchTypeName = TouchSensorTypeSupport.get_type_name(); 
            TouchSensorTypeSupport.register_type(participant, touchTypeName);
    
            /* To customize topic QoS, use
               the configuration file USER_QOS_PROFILES.xml */
    
            touchTopic = participant.create_topic(
                "TouchTopic",
                touchTypeName, DomainParticipant.TOPIC_QOS_DEFAULT,
                null /* listener */, StatusKind.STATUS_MASK_NONE);
            if (touchTopic == null) {
                statusMessage.append("create_topic error\n");
                return;
            }    
                        
            // --- Create reader --- //
            touchListener = new TouchSensorListener();
    
            /* To customize data reader QoS, use
               the configuration file USER_QOS_PROFILES.xml */
    
            touchReader = (TouchSensorDataReader)
                subscriber.create_datareader(
                    touchTopic, Subscriber.DATAREADER_QOS_DEFAULT, touchListener,
                    StatusKind.STATUS_MASK_ALL);
            if (touchReader == null) {
                statusMessage.append("create_datareader error\n");
                return;
            } 
            
            
            // --- Create topic --- //
            
            /* Register type before creating topic */
            String distanceTypeName = UltraSonicSensorTypeSupport.get_type_name(); 
            UltraSonicSensorTypeSupport.register_type(participant, distanceTypeName);
    
            /* To customize topic QoS, use
               the configuration file USER_QOS_PROFILES.xml */
    
            distanceTopic = participant.create_topic(
                "UltraSonicTopic",
                distanceTypeName, DomainParticipant.TOPIC_QOS_DEFAULT,
                null /* listener */, StatusKind.STATUS_MASK_NONE);
            if (distanceTopic == null) {
                statusMessage.append("create_topic error\n");
                return;
            }      
            
            // --- Create reader --- //
            distanceListener = new UltraSonicSensorListener();
    
            /* To customize data reader QoS, use
               the configuration file USER_QOS_PROFILES.xml */
    
            distanceReader = (UltraSonicSensorDataReader)
                subscriber.create_datareader(
                    distanceTopic, Subscriber.DATAREADER_QOS_DEFAULT, distanceListener,
                    StatusKind.STATUS_MASK_ALL);
            if (distanceReader == null) {
                statusMessage.append("create_datareader error\n");
                return;
            }                         
        
            
                
        } catch (Exception e) {		
        	e.printStackTrace();
        }
	}

	public void initUI() {
		shlRtiLegoMindstorms.setLayout(null);
		// The GUI only needs 3 things: A button to show whether the touch
		// sensor button is pressed,
		// a text label to show the ultra sonic distance, and 3 buttons for
		// sending motor commands
		// to go forward, backward, and brake.

		touchLabel = new Label(shlRtiLegoMindstorms, 0);
		touchLabel.setFont(SWTResourceManager.getFont("Segoe UI", 14, SWT.NORMAL));
		touchLabel.setBounds(366, 65, 249, 22);
		touchLabel.setText("Touch Sensor Pressed: False");

		distanceLabel = new Label(shlRtiLegoMindstorms, 0);
		distanceLabel.setFont(SWTResourceManager.getFont("Segoe UI", 14, SWT.NORMAL));
		distanceLabel.setBounds(366, 112, 325, 30);
		distanceLabel.setText("Ultrasonic Sensor Distance: 0");

		brakeButton = new Button(shlRtiLegoMindstorms, SWT.PUSH);
		brakeButton.setImage(SWTResourceManager.getImage(LegoDemo.class, "/com/rti/demo/ferrari-f12-pedals_brake.png"));
		brakeButton.setBounds(318, 233, 100, 97);

		forwardButton = new Button(shlRtiLegoMindstorms, SWT.PUSH);
		forwardButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent arg0) {
			}
		});
		forwardButton.setImage(SWTResourceManager.getImage(LegoDemo.class, "/com/rti/demo/ferrari-f12-pedals_gas.png"));
		forwardButton.setBounds(434, 234, 53, 95);

		reverseButton = new Button(shlRtiLegoMindstorms, SWT.PUSH);
		reverseButton.setFont(SWTResourceManager.getFont("Segoe UI", 40, SWT.BOLD));
		reverseButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent arg0) {
			}
		});
		reverseButton.setBounds(501, 233, 84, 97);
		reverseButton.setText("R");
		
		leftButton = new Button(shlRtiLegoMindstorms, SWT.PUSH);
		leftButton.setImage(SWTResourceManager.getImage(LegoDemo.class, "/com/rti/demo/leftTurnTrans.png"));
		leftButton.setBounds(591, 233, 100, 97);
		
		rightButton = new Button(shlRtiLegoMindstorms, SWT.PUSH);
		rightButton.setImage(SWTResourceManager.getImage(LegoDemo.class, "/com/rti/demo/rightTurnTrans.png"));
		rightButton.setBounds(697, 233, 99, 97);

		statusMessage = new Text(shlRtiLegoMindstorms, SWT.MULTI | SWT.BORDER | SWT.WRAP
				| SWT.V_SCROLL);
		statusMessage.setBounds(318, 375, 478, 77);
		statusMessage.append("Log Message: Nothing ");
		// statusMessage.setLayoutData(new GridData(GridData.FILL_BOTH));
				

	}

	public static void main(String[] args) {
		Display display = new Display();
		new LegoDemo(display);
		display.dispose();
		System.exit(0);
	}


	private static class TouchSensorListener extends DataReaderAdapter {
	    
	    TouchSensorSeq _dataSeq = new TouchSensorSeq();
	    SampleInfoSeq _infoSeq = new SampleInfoSeq();
	
	    public void on_data_available(DataReader reader) {
	        TouchSensorDataReader TouchSensorReader =
	            (TouchSensorDataReader)reader;
	        
	        try {
	            TouchSensorReader.take(
	                _dataSeq, _infoSeq,
	                ResourceLimitsQosPolicy.LENGTH_UNLIMITED,
	                SampleStateKind.ANY_SAMPLE_STATE,
	                ViewStateKind.ANY_VIEW_STATE,
	                InstanceStateKind.ANY_INSTANCE_STATE);
	
	            for(int i = 0; i < _dataSeq.size(); ++i) {
	                SampleInfo info = (SampleInfo)_infoSeq.get(i);
	
	                if (info.valid_data) {
	                    /*System.out.println(
	                        ((TouchSensor)_dataSeq.get(i)).toString("Received",0));*/
	                	
	                	if(((TouchSensor)_dataSeq.get(i)).value) {
		                	Display.getDefault().asyncExec(new Runnable() {
		                		public void run() {
		                			if(touchLabel != null) {
		                				if (touchLabel.isDisposed ()) return;
		                				touchLabel.setText("Touch Sensor Pressed: TRUE!");
		                			}
		                		}
		                	});
	                	} else {
	                		Display.getDefault().asyncExec(new Runnable() {
		                		public void run() {
		                			if(touchLabel != null) {
		                				if (touchLabel.isDisposed ()) return;
		                				touchLabel.setText("Touch Sensor Pressed: False.");
		                			}
		                		}
		                	});
	                	}	                		                
	                }
	            }
	        } catch (RETCODE_NO_DATA noData) {
	            // No data to process
	        } finally {
	            TouchSensorReader.return_loan(_dataSeq, _infoSeq);
	        }
	    }
	}
	
	private static class UltraSonicSensorListener extends DataReaderAdapter {
        
        UltraSonicSensorSeq _dataSeq = new UltraSonicSensorSeq();
        SampleInfoSeq _infoSeq = new SampleInfoSeq();

        public void on_data_available(DataReader reader) {
            UltraSonicSensorDataReader UltraSonicSensorReader =
                (UltraSonicSensorDataReader)reader;
            
            try {
                UltraSonicSensorReader.take(
                    _dataSeq, _infoSeq,
                    ResourceLimitsQosPolicy.LENGTH_UNLIMITED,
                    SampleStateKind.ANY_SAMPLE_STATE,
                    ViewStateKind.ANY_VIEW_STATE,
                    InstanceStateKind.ANY_INSTANCE_STATE);

                for(int i = 0; i < _dataSeq.size(); ++i) {
                    SampleInfo info = (SampleInfo)_infoSeq.get(i);

                    if (info.valid_data) {
                        //System.out.println(
                          //  ((UltraSonicSensor)_dataSeq.get(i)).toString("Received",0));                                            	
		                Display.getDefault().asyncExec(new DistanceUpdateThread(((UltraSonicSensor)_dataSeq.get(i)).value));	                	
                    }
                }
            } catch (RETCODE_NO_DATA noData) {
                // No data to process
            } finally {
                UltraSonicSensorReader.return_loan(_dataSeq, _infoSeq);
            }
        }
    }		
	
	private static class DistanceUpdateThread implements Runnable {
		double distance; 
		public DistanceUpdateThread (double newDistance) {
			distance = newDistance;
		}
		
		public void run() {
			try {
				
				if (distanceLabel.isDisposed ()) return;
				distanceLabel.setText("Ultrasonic Sensor Distance: " + distance);
				
				//The distance from the Lego goes from 0 - 255, convert this into a percent so it 255 is 100% of the progress bar and 0 = 0.
				if (distanceProgressBar.isDisposed ()) return;
				distanceProgressBar.setSelection((int)(distance / 255 * 100));
			} catch (org.eclipse.swt.SWTException e) {
				System.out.println("SWT Exception caught.");
				e.printStackTrace();
			} catch (Exception e) {
				e.printStackTrace();
			}
			
		}
	}
}
